@@ -1,9 +1,9 @@
 /*
  * libpri: An implementation of Primary Rate ISDN
  *
- * Written by Mark Spencer <markster@linux-support.net>
+ * Written by Mark Spencer <markster@digium.com>
  *
- * Copyright (C) 2001, Linux Support Services, Inc.
+ * Copyright (C) 2001-2005, Digium
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -21,12 +21,13 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
  *
  */
- 
+
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <errno.h>
+#include "compat.h"
 #include "libpri.h"
 #include "pri_internal.h"
 #include "pri_q921.h" 
@@ -71,7 +72,7 @@ static int q921_transmit(struct pri *pri, q921_h *h, int len)
 		return q921_transmit(pri->master, h, len);
 #ifdef RANDOM_DROPS
    if (!(random() % 3)) {
-         pri_message(" === Dropping Packet ===\n");
+         pri_message(pri, " === Dropping Packet ===\n");
          return 0;
    }
 #endif   
@@ -80,11 +81,11 @@ static int q921_transmit(struct pri *pri, q921_h *h, int len)
 #endif
 	/* Just send it raw */
 	if (pri->debug & PRI_DEBUG_Q921_DUMP)
-		q921_dump(h, len, pri->debug & PRI_DEBUG_Q921_RAW, 1);
+		q921_dump(pri, h, len, pri->debug & PRI_DEBUG_Q921_RAW, 1);
 	/* Write an extra two bytes for the FCS */
-	res = write(pri->fd, h, len + 2);
+	res = pri->write_func ? pri->write_func(pri, h, len + 2) : 0;
 	if (res != (len + 2)) {
-		pri_error("Short write: %d/%d (%s)\n", res, len + 2, strerror(errno));
+		pri_error(pri, "Short write: %d/%d (%s)\n", res, len + 2, strerror(errno));
 		return -1;
 	}
 	reschedule_t203(pri);
@@ -107,11 +108,11 @@ static void q921_send_ua(struct pri *pri, int pfbit)
 		h.h.c_r = 1;
 		break;
 	default:
-		pri_error("Don't know how to U/A on a type %d node\n", pri->localtype);
+		pri_error(pri, "Don't know how to U/A on a type %d node\n", pri->localtype);
 		return;
 	}
 	if (pri->debug & PRI_DEBUG_Q921_STATE)
-		pri_message("Sending Unnumbered Acknowledgement\n");
+		pri_message(pri, "Sending Unnumbered Acknowledgement\n");
 	q921_transmit(pri, &h, 3);
 }
 
@@ -123,7 +124,7 @@ static void q921_send_sabme(void *vpri, int now)
 	q921_h h;
 	pri_schedule_del(pri, pri->sabme_timer);
 	pri->sabme_timer = 0;
-	pri->sabme_timer = pri_schedule_event(pri, T_200, q921_send_sabme_now, pri);
+	pri->sabme_timer = pri_schedule_event(pri, pri->timers[PRI_TIMER_T200], q921_send_sabme_now, pri);
 	if (!now)
 		return;
 	Q921_INIT(pri, h);
@@ -139,11 +140,11 @@ static void q921_send_sabme(void *vpri, int now)
 		h.h.c_r = 0;
 		break;
 	default:
-		pri_error("Don't know how to U/A on a type %d node\n", pri->localtype);
+		pri_error(pri, "Don't know how to U/A on a type %d node\n", pri->localtype);
 		return;
 	}
 	if (pri->debug & PRI_DEBUG_Q921_STATE)
-		pri_message("Sending Set Asynchronous Balanced Mode Extended\n");
+		pri_message(pri, "Sending Set Asynchronous Balanced Mode Extended\n");
 	q921_transmit(pri, &h, 3);
 	pri->q921_state = Q921_AWAITING_ESTABLISH;
 }
@@ -166,7 +167,7 @@ static int q921_ack_packet(struct pri *pri, int num)
 			else
 				pri->txqueue = f->next;
 			if (pri->debug & PRI_DEBUG_Q921_STATE)
-				pri_message("-- ACKing packet %d, new txqueue is %d (-1 means empty)\n", f->h.n_s, pri->txqueue ? pri->txqueue->h.n_s : -1);
+				pri_message(pri, "-- ACKing packet %d, new txqueue is %d (-1 means empty)\n", f->h.n_s, pri->txqueue ? pri->txqueue->h.n_s : -1);
 			/* Update v_a */
 			pri->v_a = num;
 			free(f);
@@ -180,7 +181,7 @@ static int q921_ack_packet(struct pri *pri, int num)
 				if (!f->transmitted) {
 					/* Send it now... */
 					if (pri->debug & PRI_DEBUG_Q921_STATE)
-						pri_message("-- Finally transmitting %d, since window opened up\n", f->h.n_s);
+						pri_message(pri, "-- Finally transmitting %d, since window opened up\n", f->h.n_s);
 					f->transmitted++;
 					pri->windowlen++;
 					f->h.n_r = pri->v_r;
@@ -206,9 +207,9 @@ static void reschedule_t203(struct pri *pri)
 	if (pri->t203_timer) {
 		pri_schedule_del(pri, pri->t203_timer);
 		if (pri->debug &  PRI_DEBUG_Q921_STATE)
-			pri_message("-- Restarting T203 counter\n");
+			pri_message(pri, "-- Restarting T203 counter\n");
 		/* Nothing to transmit, start the T203 counter instead */
-		pri->t203_timer = pri_schedule_event(pri, T_203, t203_expire, pri);
+		pri->t203_timer = pri_schedule_event(pri, pri->timers[PRI_TIMER_T203], t203_expire, pri);
 	}
 }
 
@@ -221,7 +222,7 @@ static pri_event *q921_ack_rx(struct pri *pri, int ack)
 	for (x=pri->v_a; (x != pri->v_s) && (x != ack); Q921_INC(x));
 	if (x != ack) {
 		/* ACK was outside of our window --- ignore */
-		pri_error("ACK received for '%d' outside of window of '%d' to '%d', restarting\n", ack, pri->v_a, pri->v_s);
+		pri_error(pri, "ACK received for '%d' outside of window of '%d' to '%d', restarting\n", ack, pri->v_a, pri->v_s);
 		ev = q921_dchannel_down(pri);
 		q921_start(pri, 1);
 		pri->schedev = 1;
@@ -229,33 +230,33 @@ static pri_event *q921_ack_rx(struct pri *pri, int ack)
 	}
 	/* Cancel each packet as necessary */
 	if (pri->debug & PRI_DEBUG_Q921_STATE)
-		pri_message("-- ACKing all packets from %d to (but not including) %d\n", pri->v_a, ack);
+		pri_message(pri, "-- ACKing all packets from %d to (but not including) %d\n", pri->v_a, ack);
 	for (x=pri->v_a; x != ack; Q921_INC(x)) 
 		cnt += q921_ack_packet(pri, x);	
 	if (!pri->txqueue) {
 		if (pri->debug &  PRI_DEBUG_Q921_STATE)
-			pri_message("-- Since there was nothing left, stopping T200 counter\n");
+			pri_message(pri, "-- Since there was nothing left, stopping T200 counter\n");
 		/* Something was ACK'd.  Stop T200 counter */
 		pri_schedule_del(pri, pri->t200_timer);
 		pri->t200_timer = 0;
 	}
 	if (pri->t203_timer) {
 		if (pri->debug &  PRI_DEBUG_Q921_STATE)
-			pri_message("-- Stopping T203 counter since we got an ACK\n");
+			pri_message(pri, "-- Stopping T203 counter since we got an ACK\n");
 		pri_schedule_del(pri, pri->t203_timer);
 		pri->t203_timer = 0;
 	}
 	if (pri->txqueue) {
 		/* Something left to transmit, Start the T200 counter again if we stopped it */
 		if (pri->debug &  PRI_DEBUG_Q921_STATE)
-			pri_message("-- Something left to transmit (%d), restarting T200 counter\n", pri->txqueue->h.n_s);
+			pri_message(pri, "-- Something left to transmit (%d), restarting T200 counter\n", pri->txqueue->h.n_s);
 		if (!pri->t200_timer)
-			pri->t200_timer = pri_schedule_event(pri, T_200, t200_expire, pri);
+			pri->t200_timer = pri_schedule_event(pri, pri->timers[PRI_TIMER_T200], t200_expire, pri);
 	} else {
 		if (pri->debug &  PRI_DEBUG_Q921_STATE)
-			pri_message("-- Nothing left, starting T203 counter\n");
+			pri_message(pri, "-- Nothing left, starting T203 counter\n");
 		/* Nothing to transmit, start the T203 counter instead */
-		pri->t203_timer = pri_schedule_event(pri, T_203, t203_expire, pri);
+		pri->t203_timer = pri_schedule_event(pri, pri->timers[PRI_TIMER_T203], t203_expire, pri);
 	}
 	return NULL;
 }
@@ -277,11 +278,11 @@ static void q921_reject(struct pri *pri, int pf)
 		h.h.c_r = 1;
 		break;
 	default:
-		pri_error("Don't know how to U/A on a type %d node\n", pri->localtype);
+		pri_error(pri, "Don't know how to U/A on a type %d node\n", pri->localtype);
 		return;
 	}
 	if (pri->debug & PRI_DEBUG_Q921_STATE)
-		pri_message("Sending Reject (%d)\n", pri->v_r);
+		pri_message(pri, "Sending Reject (%d)\n", pri->v_r);
 	pri->sentrej = 1;
 	q921_transmit(pri, &h, 4);
 }
@@ -308,12 +309,12 @@ static void q921_rr(struct pri *pri, int pbit, int cmd) {
 			h.h.c_r = 1;
 		break;
 	default:
-		pri_error("Don't know how to U/A on a type %d node\n", pri->localtype);
+		pri_error(pri, "Don't know how to U/A on a type %d node\n", pri->localtype);
 		return;
 	}
 	pri->v_na = pri->v_r;	/* Make a note that we've already acked this */
 	if (pri->debug & PRI_DEBUG_Q921_STATE)
-		pri_message("Sending Receiver Ready (%d)\n", pri->v_r);
+		pri_message(pri, "Sending Receiver Ready (%d)\n", pri->v_r);
 	q921_transmit(pri, &h, 4);
 }
 
@@ -323,7 +324,7 @@ static void t200_expire(void *vpri)
 	if (pri->txqueue) {
 		/* Retransmit first packet in the queue, setting the poll bit */
 		if (pri->debug & PRI_DEBUG_Q921_STATE)
-			pri_message("-- T200 counter expired, What to do...\n");
+			pri_message(pri, "-- T200 counter expired, What to do...\n");
 		/* Force Poll bit */
 		pri->txqueue->h.p_f = 1;	
 		/* Update nr */
@@ -332,23 +333,23 @@ static void t200_expire(void *vpri)
 		pri->solicitfbit = 1;
 		pri->retrans++;
       /* Up to three retransmissions */
-      if (pri->retrans < N_200) {
+      if (pri->retrans < pri->timers[PRI_TIMER_N200]) {
          /* Reschedule t200_timer */
          if (pri->debug & PRI_DEBUG_Q921_STATE)
-            pri_message("-- Retransmitting %d bytes\n", pri->txqueue->len);
+            pri_message(pri, "-- Retransmitting %d bytes\n", pri->txqueue->len);
 		if (pri->busy) 
-			q921_rr(pri, 1, 0);
+			q921_rr(pri, 1, 1);
 		else {
 			if (!pri->txqueue->transmitted) 
-				pri_error("!! Not good - head of queue has not been transmitted yet\n");
+				pri_error(pri, "!! Not good - head of queue has not been transmitted yet\n");
 			q921_transmit(pri, (q921_h *)&pri->txqueue->h, pri->txqueue->len);
 		}
          if (pri->debug & PRI_DEBUG_Q921_STATE) 
-               pri_message("-- Rescheduling retransmission (%d)\n", pri->retrans);
-         pri->t200_timer = pri_schedule_event(pri, T_200, t200_expire, pri);
+               pri_message(pri, "-- Rescheduling retransmission (%d)\n", pri->retrans);
+         pri->t200_timer = pri_schedule_event(pri, pri->timers[PRI_TIMER_T200], t200_expire, pri);
       } else {
          if (pri->debug & PRI_DEBUG_Q921_STATE) 
-               pri_message("-- Timeout occured, restarting PRI\n");
+               pri_message(pri, "-- Timeout occured, restarting PRI\n");
          pri->q921_state = Q921_LINK_CONNECTION_RELEASED;
       	pri->t200_timer = 0;
          q921_dchannel_down(pri);
@@ -357,15 +358,15 @@ static void t200_expire(void *vpri)
       }
 	} else if (pri->solicitfbit) {
          if (pri->debug & PRI_DEBUG_Q921_STATE)
-            pri_message("-- Retrying poll with f-bit\n");
+            pri_message(pri, "-- Retrying poll with f-bit\n");
 		pri->retrans++;
-		if (pri->retrans < N_200) {
+		if (pri->retrans < pri->timers[PRI_TIMER_N200]) {
 			pri->solicitfbit = 1;
 			q921_rr(pri, 1, 1);
-			pri->t200_timer = pri_schedule_event(pri, T_200, t200_expire, pri);
+			pri->t200_timer = pri_schedule_event(pri, pri->timers[PRI_TIMER_T200], t200_expire, pri);
 		} else {
 			if (pri->debug & PRI_DEBUG_Q921_STATE) 
-				pri_message("-- Timeout occured, restarting PRI\n");
+				pri_message(pri, "-- Timeout occured, restarting PRI\n");
 			pri->q921_state = Q921_LINK_CONNECTION_RELEASED;
 			pri->t200_timer = 0;
 			q921_dchannel_down(pri);
@@ -373,7 +374,7 @@ static void t200_expire(void *vpri)
 			pri->schedev = 1;
 		}
 	} else {
-		pri_error("T200 counter expired, nothing to send...\n");
+		pri_error(pri, "T200 counter expired, nothing to send...\n");
 	   	pri->t200_timer = 0;
 	}
 }
@@ -423,26 +424,26 @@ int q921_transmit_iframe(struct pri *pri, void *buf, int len, int cr)
 				f->transmitted++;
 			} else {
 				if (pri->debug & PRI_DEBUG_Q921_STATE)
-					pri_message("Delaying transmission of %d, window is %d/%d long\n", 
+					pri_message(pri, "Delaying transmission of %d, window is %d/%d long\n", 
 						f->h.n_s, pri->windowlen, pri->window);
 			}
 		}
 		if (pri->t203_timer) {
 			if (pri->debug & PRI_DEBUG_Q921_STATE)
-				pri_message("Stopping T_203 timer\n");
+				pri_message(pri, "Stopping T_203 timer\n");
 			pri_schedule_del(pri, pri->t203_timer);
 			pri->t203_timer = 0;
 		}
 		if (!pri->t200_timer) {
 			if (pri->debug & PRI_DEBUG_Q921_STATE)
-				pri_message("Starting T_200 timer\n");
-			pri->t200_timer = pri_schedule_event(pri, T_200, t200_expire, pri);
+				pri_message(pri, "Starting T_200 timer\n");
+			pri->t200_timer = pri_schedule_event(pri, pri->timers[PRI_TIMER_T200], t200_expire, pri);
 		} else
 			if (pri->debug & PRI_DEBUG_Q921_STATE)
-				pri_message("T_200 timer already going (%d)\n", pri->t200_timer);
+				pri_message(pri, "T_200 timer already going (%d)\n", pri->t200_timer);
 		
 	} else {
-		pri_error("!! Out of memory for Q.921 transmit\n");
+		pri_error(pri, "!! Out of memory for Q.921 transmit\n");
 		return -1;
 	}
 	return 0;
@@ -453,16 +454,16 @@ static void t203_expire(void *vpri)
 	struct pri *pri = vpri;
 	if (pri->q921_state == Q921_LINK_CONNECTION_ESTABLISHED) {
 		if (pri->debug &  PRI_DEBUG_Q921_STATE)
-			pri_message("T203 counter expired, sending RR and scheduling T203 again\n");
+			pri_message(pri, "T203 counter expired, sending RR and scheduling T203 again\n");
 		/* Solicit an F-bit in the other's RR */
 		pri->solicitfbit = 1;
 		pri->retrans = 0;
 		q921_rr(pri, 1, 1);
 		/* Start timer T200 to resend our RR if we don't get it */
-		pri->t203_timer = pri_schedule_event(pri, T_200, t200_expire, pri);
+		pri->t203_timer = pri_schedule_event(pri, pri->timers[PRI_TIMER_T200], t200_expire, pri);
 	} else {
 		if (pri->debug &  PRI_DEBUG_Q921_STATE)
-			pri_message("T203 counter expired in weird state %d\n", pri->q921_state);
+			pri_message(pri, "T203 counter expired in weird state %d\n", pri->q921_state);
 		pri->t203_timer = 0;
 	}
 }
@@ -507,7 +508,7 @@ static pri_event *q921_handle_iframe(struct pri *pri, q921_i *i, int len)
 	return NULL;
 }
 
-void q921_dump(q921_h *h, int len, int showraw, int txrx)
+void q921_dump(struct pri *pri, q921_h *h, int len, int showraw, int txrx)
 {
 	int x;
         char *type;
@@ -520,7 +521,7 @@ void q921_dump(q921_h *h, int len, int showraw, int txrx)
 		if (buf) {
 			for (x=0;x<len;x++) 
 				buflen += sprintf(buf + buflen, "%02x ", h->raw[x]);
-			pri_message("\n%c [ %s]\n", direction_tag, buf);
+			pri_message(pri, "\n%c [ %s]\n", direction_tag, buf);
 			free(buf);
 		}
 	}
@@ -528,17 +529,17 @@ void q921_dump(q921_h *h, int len, int showraw, int txrx)
 	switch (h->h.data[0] & Q921_FRAMETYPE_MASK) {
 	case 0:
 	case 2:
-		pri_message("\n%c Informational frame:\n", direction_tag);
+		pri_message(pri, "\n%c Informational frame:\n", direction_tag);
 		break;
 	case 1:
-		pri_message("\n%c Supervisory frame:\n", direction_tag);
+		pri_message(pri, "\n%c Supervisory frame:\n", direction_tag);
 		break;
 	case 3:
-		pri_message("\n%c Unnumbered frame:\n", direction_tag);
+		pri_message(pri, "\n%c Unnumbered frame:\n", direction_tag);
 		break;
 	}
 	
-	pri_message(
+	pri_message(pri, 
 "%c SAPI: %02d  C/R: %d EA: %d\n"
 "%c  TEI: %03d        EA: %d\n", 
     	direction_tag,
@@ -552,7 +553,7 @@ void q921_dump(q921_h *h, int len, int showraw, int txrx)
 	case 0:
 	case 2:
 		/* Informational frame */
-		pri_message(
+		pri_message(pri, 
 "%c N(S): %03d   0: %d\n"
 "%c N(R): %03d   P: %d\n"
 "%c %d bytes of data\n",
@@ -579,7 +580,7 @@ void q921_dump(q921_h *h, int len, int showraw, int txrx)
 			type = "REJ (reject)";
 			break;
 		}
-		pri_message(
+		pri_message(pri, 
 "%c Zero: %d     S: %d 01: %d  [ %s ]\n"
 "%c N(R): %03d P/F: %d\n"
 "%c %d bytes of data\n",
@@ -625,7 +626,7 @@ void q921_dump(q921_h *h, int len, int showraw, int txrx)
 				break;
 			}
 		}
-		pri_message(
+		pri_message(pri, 
 "%c   M3: %d   P/F: %d M2: %d 11: %d  [ %s ]\n"
 "%c %d bytes of data\n",
 	direction_tag,
@@ -656,7 +657,7 @@ static pri_event *q921_dchannel_up(struct pri *pri)
 	pri->q921_state = Q921_LINK_CONNECTION_ESTABLISHED;
 
 	/* Start the T203 timer */
-	pri->t203_timer = pri_schedule_event(pri, T_203, t203_expire, pri);
+	pri->t203_timer = pri_schedule_event(pri, pri->timers[PRI_TIMER_T203], t203_expire, pri);
 	
 	/* Report event that D-Channel is now up */
 	pri->ev.gen.e = PRI_EVENT_DCHAN_UP;
@@ -680,7 +681,7 @@ void q921_reset(struct pri *pri)
 	pri->v_a = 0;
 	pri->v_r = 0;
 	pri->v_na = 0;
-	pri->window = 7;
+	pri->window = pri->timers[PRI_TIMER_K];
 	pri->windowlen = 0;
 	pri_schedule_del(pri, pri->sabme_timer);
 	pri_schedule_del(pri, pri->t203_timer);
@@ -708,23 +709,23 @@ static pri_event *__q921_receive_qualified(struct pri *pri, q921_h *h, int len)
 	case 0:
 	case 2:
 		if (pri->q921_state != Q921_LINK_CONNECTION_ESTABLISHED) {
-			pri_error("!! Got I-frame while link state %d\n", pri->q921_state);
+			pri_error(pri, "!! Got I-frame while link state %d\n", pri->q921_state);
 			return NULL;
 		}
 		/* Informational frame */
 		if (len < 4) {
-			pri_error("!! Received short I-frame (expected 4, got %d)\n", len);
+			pri_error(pri, "!! Received short I-frame (expected 4, got %d)\n", len);
 			break;
 		}
 		return q921_handle_iframe(pri, &h->i, len);	
 		break;
 	case 1:
 		if (pri->q921_state != Q921_LINK_CONNECTION_ESTABLISHED) {
-			pri_error("!! Got S-frame while link down\n");
+			pri_error(pri, "!! Got S-frame while link down\n");
 			return NULL;
 		}
 		if (len < 4) {
-			pri_error("!! Received short S-frame (expected 4, got %d)\n", len);
+			pri_error(pri, "!! Received short S-frame (expected 4, got %d)\n", len);
 			break;
 		}
 		switch(h->s.ss) {
@@ -739,10 +740,10 @@ static pri_event *__q921_receive_qualified(struct pri *pri, q921_h *h, int len)
 				/* If it's a p/f one then send back a RR in return with the p/f bit set */
 				if (pri->solicitfbit) {
 					if (pri->debug & PRI_DEBUG_Q921_STATE) 
-						pri_message("-- Got RR response to our frame\n");
+						pri_message(pri, "-- Got RR response to our frame\n");
 				} else {
 					if (pri->debug & PRI_DEBUG_Q921_STATE) 
-						pri_message("-- Unsolicited RR with P/F bit, responding\n");
+						pri_message(pri, "-- Unsolicited RR with P/F bit, responding\n");
 						q921_rr(pri, 1, 0);
 				}
 				pri->solicitfbit = 0;
@@ -751,7 +752,7 @@ static pri_event *__q921_receive_qualified(struct pri *pri, q921_h *h, int len)
       case 1:
          /* Receiver not ready */
          if (pri->debug & PRI_DEBUG_Q921_STATE)
-            pri_message("-- Got receiver not ready\n");
+            pri_message(pri, "-- Got receiver not ready\n");
 	 if(h->s.p_f) {
 		/* Send RR if poll bit set */
 		q921_rr(pri, h->s.p_f, 0);
@@ -761,7 +762,7 @@ static pri_event *__q921_receive_qualified(struct pri *pri, q921_h *h, int len)
       case 2:
          /* Just retransmit */
          if (pri->debug & PRI_DEBUG_Q921_STATE)
-            pri_message("-- Got reject requesting packet %d...  Retransmitting.\n", h->s.n_r);
+            pri_message(pri, "-- Got reject requesting packet %d...  Retransmitting.\n", h->s.n_r);
          if (h->s.p_f) {
             /* If it has the poll bit set, send an appropriate supervisory response */
             q921_rr(pri, 1, 0);
@@ -773,7 +774,7 @@ static pri_event *__q921_receive_qualified(struct pri *pri, q921_h *h, int len)
                      /* Matches the request, or follows in our window, and has
 					    already been transmitted. */
 					 sendnow = 1;
-					 pri_error("!! Got reject for frame %d, retransmitting frame %d now, updating n_r!\n", h->s.n_r, f->h.n_s);
+					 pri_error(pri, "!! Got reject for frame %d, retransmitting frame %d now, updating n_r!\n", h->s.n_r, f->h.n_s);
 				     f->h.n_r = pri->v_r;
                      q921_transmit(pri, (q921_h *)(&f->h), f->len);
                }
@@ -782,11 +783,11 @@ static pri_event *__q921_receive_qualified(struct pri *pri, q921_h *h, int len)
                if (pri->txqueue) {
                      /* This should never happen */
 		     if (!h->s.p_f || h->s.n_r) {
-			pri_error("!! Got reject for frame %d, but we only have others!\n", h->s.n_r);
+			pri_error(pri, "!! Got reject for frame %d, but we only have others!\n", h->s.n_r);
 		     }
                } else {
                      /* Hrm, we have nothing to send, but have been REJ'd.  Reset v_a, v_s, etc */
-				pri_error("!! Got reject for frame %d, but we have nothing -- resetting!\n", h->s.n_r);
+				pri_error(pri, "!! Got reject for frame %d, but we have nothing -- resetting!\n", h->s.n_r);
                      pri->v_a = h->s.n_r;
                      pri->v_s = h->s.n_r;
                      /* Reset t200 timer if it was somehow going */
@@ -797,18 +798,18 @@ static pri_event *__q921_receive_qualified(struct pri *pri, q921_h *h, int len)
                      /* Reset and restart t203 timer */
                      if (pri->t203_timer)
                            pri_schedule_del(pri, pri->t203_timer);
-                     pri->t203_timer = pri_schedule_event(pri, T_203, t203_expire, pri);
+                     pri->t203_timer = pri_schedule_event(pri, pri->timers[PRI_TIMER_T203], t203_expire, pri);
                }
          }
          break;
 		default:
-			pri_error("!! XXX Unknown Supervisory frame ss=0x%02x,pf=%02xnr=%02x vs=%02x, va=%02x XXX\n", h->s.ss, h->s.p_f, h->s.n_r,
+			pri_error(pri, "!! XXX Unknown Supervisory frame ss=0x%02x,pf=%02xnr=%02x vs=%02x, va=%02x XXX\n", h->s.ss, h->s.p_f, h->s.n_r,
 					pri->v_s, pri->v_a);
 		}
 		break;
 	case 3:
 		if (len < 3) {
-			pri_error("!! Received short unnumbered frame\n");
+			pri_error(pri, "!! Received short unnumbered frame\n");
 			break;
 		}
 		switch(h->u.m3) {
@@ -818,7 +819,7 @@ static pri_event *__q921_receive_qualified(struct pri *pri, q921_h *h, int len)
 					/* Section 5.7.1 says we should restart on receiving a DM response with the f-bit set to
 					   one, but we wait T200 first */
 					if (pri->debug & PRI_DEBUG_Q921_STATE)
-						pri_message("-- Got DM Mode from peer.\n");
+						pri_message(pri, "-- Got DM Mode from peer.\n");
 					/* Disconnected mode, try again after T200 */
 					ev = q921_dchannel_down(pri);
 					q921_start(pri, 0);
@@ -826,7 +827,7 @@ static pri_event *__q921_receive_qualified(struct pri *pri, q921_h *h, int len)
 						
 				} else {
 					if (pri->debug & PRI_DEBUG_Q921_STATE)
-						pri_message("-- Ignoring unsolicited DM with p/f set to 0\n");
+						pri_message(pri, "-- Ignoring unsolicited DM with p/f set to 0\n");
 #if 0
 					/* Requesting that we start */
 					q921_start(pri, 0);
@@ -834,12 +835,12 @@ static pri_event *__q921_receive_qualified(struct pri *pri, q921_h *h, int len)
 				}
 				break;
 			} else if (!h->u.m2) {
-				pri_message("XXX Unnumbered Information not implemented XXX\n");
+				pri_message(pri, "XXX Unnumbered Information not implemented XXX\n");
 			}
 			break;
 		case 2:
 			if (pri->debug &  PRI_DEBUG_Q921_STATE)
-				pri_message("-- Got Disconnect from peer.\n");
+				pri_message(pri, "-- Got Disconnect from peer.\n");
 			/* Acknowledge */
 			q921_send_ua(pri, h->u.p_f);
 			ev = q921_dchannel_down(pri);
@@ -849,7 +850,7 @@ static pri_event *__q921_receive_qualified(struct pri *pri, q921_h *h, int len)
 			if (h->u.m2 == 3) {
 				/* SABME */
 				if (pri->debug & PRI_DEBUG_Q921_STATE) {
-					pri_message("-- Got SABME from %s peer.\n", h->h.c_r ? "network" : "cpe");
+					pri_message(pri, "-- Got SABME from %s peer.\n", h->h.c_r ? "network" : "cpe");
 				}
 				if (h->h.c_r) {
 					pri->remotetype = PRI_NETWORK;
@@ -871,22 +872,22 @@ static pri_event *__q921_receive_qualified(struct pri *pri, q921_h *h, int len)
 					/* It's a UA */
 				if (pri->q921_state == Q921_AWAITING_ESTABLISH) {
 					if (pri->debug & PRI_DEBUG_Q921_STATE) {
-						pri_message("-- Got UA from %s peer  Link up.\n", h->h.c_r ? "cpe" : "network");
+						pri_message(pri, "-- Got UA from %s peer  Link up.\n", h->h.c_r ? "cpe" : "network");
 					}
 					return q921_dchannel_up(pri);
 				} else 
-					pri_error("!! Got a UA, but i'm in state %d\n", pri->q921_state);
+					pri_error(pri, "!! Got a UA, but i'm in state %d\n", pri->q921_state);
 			} else 
-				pri_error("!! Weird frame received (m3=3, m2 = %d)\n", h->u.m2);
+				pri_error(pri, "!! Weird frame received (m3=3, m2 = %d)\n", h->u.m2);
 			break;
 		case 4:
-			pri_error("!! Frame got rejected!\n");
+			pri_error(pri, "!! Frame got rejected!\n");
 			break;
 		case 5:
-			pri_error("!! XID frames not supported\n");
+			pri_error(pri, "!! XID frames not supported\n");
 			break;
 		default:
-			pri_error("!! Don't know what to do with M3=%d u-frames\n", h->u.m3);
+			pri_error(pri, "!! Don't know what to do with M3=%d u-frames\n", h->u.m3);
 		}
 		break;
 				
@@ -901,7 +902,7 @@ static pri_event *__q921_receive(struct pri *pri, q921_h *h, int len)
 	len -= 2;
 	
 	if (!pri->master && pri->debug & PRI_DEBUG_Q921_DUMP)
-		q921_dump(h, len, pri->debug & PRI_DEBUG_Q921_RAW, 0);
+		q921_dump(pri, h, len, pri->debug & PRI_DEBUG_Q921_RAW, 0);
 
 	/* Check some reject conditions -- Start by rejecting improper ea's */
 	if (h->h.ea1 || !(h->h.ea2))
@@ -940,7 +941,7 @@ pri_event *q921_receive(struct pri *pri, q921_h *h, int len)
 void q921_start(struct pri *pri, int now)
 {
 	if (pri->q921_state != Q921_LINK_CONNECTION_RELEASED) {
-		pri_error("!! q921_start: Not in 'Link Connection Released' state\n");
+		pri_error(pri, "!! q921_start: Not in 'Link Connection Released' state\n");
 		return;
 	}
 	/* Reset our interface */
